# -*- coding: utf-8 -*-
import os
import time
import traceback
import shutil
import json
from example_demo.program_process.process_sftp import SftpProcess
from example_demo.program_process.process_ftp import FtpProcess
from example_demo.program_process.process_request import RequestProcess

import example_demo.setting as setting
import example_demo.commons.common_fun as common
from example_demo.monitor.monitor_signal_alert import send_mail, pipe_monitor_file
from example_demo.utils.redis_client import connect_redis
from example_demo.utils.jobctl import JobCtl


class ProgramProcess:
    def __init__(self, url_s=None, parser_c=None, init_dpool=False, **kwargs):

        self.debug = True if setting.DEBUG_ENABLE == '1' else False
        self.url_customize = setting.SOURCE_CUSTOMIZE
        self.request_type = setting.SOURCE_REQUEST_TYPE
        self.dpool = setting.DPOOL_ENABLE
        self.url_s = url_s
        self.parser_c = parser_c
        self.init_dpool = init_dpool
        self.setting_info = setting.setting_info

        self.init_kwargs(kwargs)

    # 初始化入参，覆盖更新配置参数
    def init_kwargs(self, kwargs):
        pass
        # url_params = self.setting_info.get("url_params")
        # for k, v in kwargs.items():
        #     if v.startswith("{") and v.endswith("}"):
        #         v_para = v[1:-1]
        #         if self.setting_info.get(v_para):
        #             url_params[k] = setting.setting_info.get(v_para)

    def run(self):

        if self.init_dpool:
            return self.init_dpool_program()

        if self.dpool:
            return self.dpool_program()

        url_source = self.parse_url_cfg() if self.url_customize != '1' else self.url_s.UrlSource(
            debug=self.debug).load_all_source()
        if self.debug:
            # print('debug_over')
            print(url_source)
            print('debug_over')

            return
        self.program_process(url_source)

    def program_process(self, url_source):
        # 这里request_type 可以细化到每个sour ,可以对应做个优化
        for sour in url_source:
            if self.request_type == 'sftp':
                self.sftp_process(sour)
            elif self.request_type == 'ftp':
                self.ftp_process(sour)
            elif self.request_type == 'api':
                self.api_process(sour)

    # 集群流程
    def init_dpool_program(self):
        all_items = self.url_s.get_dpool_init_items()
        load_db = setting.REDIS_LOAD_DB
        # redis_client = connect_redis(load_db)
        # for seeder in all_items:
        #     redis_client.lpush(self.url_s.DPOOL_KEY, json.dumps(seeder))

        with JobCtl('redis://sh-dpool') as ctl:
            ctl.run_pyfile('main.py', name=self.url_s.DPOOL_KEY,
                           files=['main.py', 'parser_sustomize.py', 'setting.py', 'project.cfg', 'url_source.py'])

    def dpool_program(self):

        #     获取队列
        dpool_key = self.url_s.DPOOL_KEY
        load_db = setting.REDIS_LOAD_DB
        redis_client = connect_redis(load_db)

        if self.debug:
            sour = {'code': '603863', 'max_date': '2022-06-06 12:46:00', 'script': None, 'request_func': None,
                    'retry': 0, 'max_retry': 3, 'task_key': 'guba_20220606'}
            sour = self.url_s.UrlSource(sour).load_source_dpool()
            self.api_process(sour)
        else:
            while redis_client.llen(dpool_key) > 0:
                try:
                    sour = redis_client.lpop(dpool_key)
                    sour = json.loads(sour)
                    sour = self.url_s.UrlSource(sour).load_source_dpool()
                    self.api_process(sour)
                    # get_data(deepcopy(seeder))
                except Exception as ex:
                    pass
                    # print('get seeder from redis fail. ', ex, download_config['redis_key'])
                    # with open(log_file, 'w') as obj:
                    #     obj.write('get seeder from redis fail. ' + str(ex) + " " + str(download_config['redis_key']))
        redis_client.close()

    def parse_url_cfg(self):
        url_source = {}
        for k, v in self.setting_info.items():
            if k.startswith("source"):
                url_source[k] = v

        url_params = url_source.get("url_params")
        url_format = url_source.get("url_format")
        if url_params:
            for k, v in url_params.items():
                if v.startswith("{") and v.endswith("}"):
                    v_para = v[1:-1]
                    if url_source.get(v_para):
                        url_params[k] = url_source.get(v_para)
        if url_params and url_format:
            url = common.parse_url_join(url_format, url_params)
            url_source["url"] = url
        return [url_source]

    def sftp_process(self, sour):
        request_info = sour.get("request_info", {})
        if not request_info:
            request_info["sftp_host"] = sour.get("sftp_host")
            request_info["sftp_user"] = sour.get("sftp_user")
            request_info["sftp_password"] = sour.get("sftp_password")
        load_path = sour.get("sftp_load_path")
        out_path = sour.get("sftp_out_path")
        load_min_filesize = setting.SOURCE_SFTP_LOAD_FILE_MINSIZE if not sour.get("min_filesize") else sour.get(
            "min_filesize")
        sftp_program_process = SftpProcess(request_info, load_path, out_path, min_filesize=load_min_filesize,
                                           debug=self.debug)
        load_res = sftp_program_process.run()
        if not load_res:
            print('%s load error' % load_path)
            return
        if self.parser_c:
            self.parser_c(load_res, sour)
        else:
            self.parse_normal_monitor_sftp(sour)

    def parse_normal_monitor_sftp(self, sour):

        need_send_email = sour.get("need_send_email")
        email_message = sour.get("email_message")
        need_unzip = sour.get("need_unzip")
        unzip_path = sour.get("unzip_path")
        sftp_out_path = sour.get("sftp_out_path")
        need_monitor_file = sour.get("need_monitor_file")
        download_date = sour.get("download_date")
        pipeline_check_key = sour.get("pipeline_check_key")
        real_output_path = sour.get("real_output_path")
        if need_send_email:
            if self.debug:
                print('send_email')
                print(email_message)
                # return
            send_mail(*email_message)

        if need_unzip and unzip_path:
            try:
                common.un_zip(sftp_out_path, unzip_path)
            except Exception as e:
                print('un_zip_error')
                traceback.print_exc()
        if need_monitor_file:
            # file_path, download_date, project_name, pipline_path, check_key_head, debug=False
            pipe_out_path = real_output_path if real_output_path else sftp_out_path
            file_size = str(os.path.getsize(sftp_out_path))
            report_info = pipeline_check_key + ' check success,file:' + pipe_out_path + ',exists'

            pipe_monitor_file(setting.PROJECT_NAME, pipeline_check_key, report_info, debug=self.debug,
                              file_size=file_size)

    def ftp_process(self, sour):
        request_info = sour.get("request_info", {})

        download_file_name = sour.get("download_file_name")
        ftp_dir = sour.get("ftp_dir")
        local_file_path = sour.get("local_file_path")
        target_path = sour.get("target_path")
        ftp_program_process = FtpProcess(request_info, ftp_dir, download_file_name, local_file_path)
        load_res = ftp_program_process.run()
        if not load_res:
            print('%s load error' % local_file_path)
            # return
        if self.parser_c:
            self.parser_c(load_res, sour)
        else:
            self.parse_normal_monitor(load_res, sour)

    def parse_normal_monitor(self, load_res, sour):
        target_path = sour.get("target_path")
        pipeline_project_name = sour.get("pipeline_project_name")
        pipeline_check_key = sour.get("pipeline_check_key")
        # local_file_path = sour.get("local_file_path")
        pip_target_path = sour.get("real_target_path") if sour.get("real_target_path") else target_path
        if not load_res:
            report_info = pipeline_check_key + '，检查不通过，文件' + pip_target_path + ',不存在'
            pipe_monitor_file(pipeline_project_name, pipeline_check_key, report_info, gs=False)
            return

        # shutil.copy(local_file_path, target_path)
        file_size = os.path.getsize(target_path)
        report_info = pipeline_check_key + '，检查通过，文件' + pip_target_path + ',已经存在'

        pipe_monitor_file(pipeline_project_name, pipeline_check_key, report_info, gs=True, file_size=file_size)

    def api_process(self, url_source):
        # 关于处理api流程，直观的期望处理给个url,直接进行抓取处理
        # 实际上的问题翻页问题，以及url中参数的问题，一般来说分为可控的参数，比如页数（自动累加）通过解析返回数据判断是否结束（包括返回的参数和传入限定的页数）
        # 再如不变的参数，可能的参数，每页数量
        page = setting.SOURCE_URL_START_PAGE
        pagesize = setting.SOURCE_URL_PAGESIZE
        url_stop_page_num = setting.SOURCE_URL_STOP_PAGE_NUM
        request_info = url_source.get("request_info", {})
        error_count = 0
        need_next = True
        parse_customize = self.parser_c.ParseCustomize()
        url_params = {}
        if page:
            url_params["page"] = int(page)
        if pagesize:
            url_params["pagesize"] = int(pagesize)
        if url_stop_page_num:
            url_stop_page_num = int(url_stop_page_num)
        # url格式化模板规则定义
        # format key 可查找到 对应的key，参数字典查找，参数字典参数直接添加
        # 索引方式  定义,可读性不强？ 忽略可读性，直接参数序列?
        # url 拼接方式
        # 目的本质上是使用固定的模板组成可用的url供解析使用
        # 加入format key 找到key,部分key可变，例如page,或者其他code_num
        # 增加可控参数的输入，来覆盖。输入--》init_kwargs,k-v格式或可直接更改setting参数
        # 这一步是模板化参数，可以读取模板自动解析进行程序，完成简单的逻辑，这种有单一模板链接的项目
        # url 连接本质上可通过k-v 组合，有k,那么找对应的k-v字典集合，k-v 字典集合
        # 直接给url_params start_time->{PROJECT_DATE};end_time->{PROJECT_DATE}
        # cfg setting 的方式。。。。。目前是转换成对应的大写变量，不太合理，外部加入使用不易，若只是提供给内部调用，；整合成字典？变量名一致。。
        # 再封装个字典（ugly_code...)？存储所有对应的变量名和值
        #     url_params["url_stop_page_num"] = int(url_stop_page_num)
        #
        if self.debug:
            print('api_process,request_info:%s,url_param:%s' % (json.dumps(request_info), json.dumps(url_params)))
        while error_count < 10 and need_next:
            try:
                url = url_source.get("url")
                if url_params:
                    url = url.format(**url_params)
                request_info["url"] = url
                if self.debug:
                    print(request_info)
                request_cli = RequestProcess(request_info, debug=self.debug)
                res_content = request_cli.request_get_res()
                if not res_content:
                    error_count += 1
                    continue
                parse_args = {res_content, page, pagesize}
                # 这里传入的参数，考虑动态化
                need_next = parse_customize.parse_customize_one(res_content, url_source, **url_params)
                if url_stop_page_num and url_params.get("page"):
                    if url_params.get("page") > url_stop_page_num:
                        break
                if need_next and url_params.get("page"):
                    url_params["page"] += 1
            except Exception as e:
                print('api_process error!')
                traceback.print_exc()
                error_count += 1
                time.sleep(10)
        file_result = parse_customize.final_parse(url_source)
        if url_source.get("pipeline_check_key") and not self.debug:
            self.parse_normal_monitor(file_result, url_source)

    def handle(self):
        pass


class BaseProgram:
    pass
